#ifndef FrameCPP_VERSION_8_FrTOCEvent_HH
#define FrameCPP_VERSION_8_FrTOCEvent_HH

#include "framecpp/Version8/STRING.hh"
#include "framecpp/Version8/GPSTime.hh"

namespace FrameCPP
{
  namespace Common
  {
    class TOCInfo;
  }

  namespace Version_8
  {
    //===================================================================
    //===================================================================
    class FrTOCEvent
    {
    public:
      typedef INT_4U	nevent_type;
      typedef STRING	name_type;
      typedef INT_4U	nTotalEvent_type;
      typedef INT_4U	gtimesEvent_type;
      typedef INT_4U	gtimenEvent_type;
      typedef REAL_4	amplitudeEvent_type;
      typedef INT_8U	positionEvent_type;

      struct event_type {
	GPSTime			GTime;
	amplitudeEvent_type	amplitudeEvent;
	positionEvent_type	positionEvent;
      };

      static const int event_type_size
      = sizeof( gtimesEvent_type )
	+ sizeof( gtimenEvent_type )
	+ sizeof( amplitudeEvent_type )
	+ sizeof( positionEvent_type )
	;

      typedef std::vector< event_type > events_container_type;
      /* typedef std::vector< event_container_type > events_container_type; */
      typedef std::map< name_type, events_container_type >
      nameEvent_container_type;

      FrTOCEvent( );

      Common::FrameSpec::size_type
      Bytes( const Common::StreamBase& Stream ) const;

      const nameEvent_container_type& GetEvent( ) const;

      void QueryEvent( const Common::TOCInfo& Info,
		       INT_4U FrameOffset,
		       INT_8U Position );

      bool operator==( const FrTOCEvent& RHS ) const;

      template< typename SE >
      static void Description( Common::Description& Desc );

    protected:
      nameEvent_container_type		m_info;

      FrTOCEvent( Common::IStream& Stream );
      void write( Common::OStream& Stream ) const;
    };

    inline Common::FrameSpec::size_type FrTOCEvent::
    Bytes( const Common::StreamBase& Stream ) const
    {
      Common::FrameSpec::size_type 
	retval = sizeof( nevent_type )
	+ sizeof( nTotalEvent_type );
      //-----------------------------------------------------------------
      // 
      //-----------------------------------------------------------------
      for ( nameEvent_container_type::const_iterator
	      cur = m_info.begin( ),
	      last = m_info.end( );
	    cur != last;
	    ++cur )
      {
	retval +=
	  cur->first.Bytes( )
	  + sizeof( nevent_type )
	  + ( cur->second.size( ) * event_type_size )
	  ;
      }
      return retval;
    }

    template< typename SE >
    void FrTOCEvent::
    Description( Common::Description& Desc )
    {
      Desc( SE( "nEventType", "INT_4U",
		"Number of FrEvent in the file" ) );
      Desc( SE( "nameEvent", "STRING[nEventType]",
		"Array of FrEvent names" ) );
      Desc( SE( "nEvent", "INT_4U[nEventType]",
		"Number of FrEvent for each type of FrEvent"
		" (size of nEventType)" ) );
      Desc( SE( "nTotalEvent", "INT_4U",
		"Total number of FrEvent" ) );
      Desc( SE( "GTimeSEvent", "INT_4U[nTotalEvent]",
		"GPS time in integer seconds" ) );
      Desc( SE( "GTimeNEvent", "INT_4U[nTotalEvent]",
		"Residual GPS time in integer nanoseconds" ) );
      Desc( SE( "amplitudeEvent", "REAL_4[nTotalEvent]",
		"Event amplitude" ) );
      Desc( SE( "positionEvent", "INT_8U[nTotalEvent]",
		"Array of FrEvent positions, in bytes,"
		" from beginning of file" ) );
    }

    inline const FrTOCEvent::nameEvent_container_type& FrTOCEvent::
    GetEvent( ) const
    {
      return m_info;
    }

  } // namespace - Version_8
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_8_FrTOCEvent_HH */
